@@ -111,6 +111,9 @@ __all__ = ['StorageInventoryClient', 'FileInfo', 'cadcput_cli', 'cadcget_cli',
 DATE_FORMAT = '%Y-%m-%dT%H:%M:%S.%f'
 # default inventory storage resource ID
 DEFAULT_RESOURCE_ID = 'ivo://cadc.nrc.ca/global/raven'
+# data resource ID - shell service backwards compatible with the initial CADC
+# data web service
+DATA_RESOURCE_ID = 'ivo://cadc.nrc.ca/data'
 
 # resource IDs for file transfers (LOCATE is for transfer negotiation
 # and FILES is for direct access)
@@ -189,16 +192,17 @@ def handle_error(exception, exit_after=True):
         sys.exit(-1)  # TODO use different error codes?
 
 
-def validate_uri(uri):
+def validate_uri(uri, strict=True):
     """
     Validate URI format for ID
     :param uri:
+    :param strict: need to include a scheme
     :return: None if uri valid or raises AttributeError otherwise
     """
     if not uri:
         raise AttributeError('URI required')
     res = urlparse(uri)
-    if not res.scheme:
+    if strict and not res.scheme:
         raise AttributeError(
             '{} not a valid id (missing URI scheme)'.format(uri))
 
@@ -210,12 +214,50 @@ def argparse_validate_uri(uri):
     :return:
     """
     try:
-        validate_uri(uri)
+        validate_uri(uri, False)
     except AttributeError as e:
         raise argparse.ArgumentTypeError(str(e))
     return uri
 
 
+def argparse_validate_uri_strict(uri):
+    """
+    Same as `validate_uri` strict but customized to be used with argparse
+    :param uri:
+    :return:
+    """
+    try:
+        validate_uri(uri, True)
+    except AttributeError as e:
+        raise argparse.ArgumentTypeError(str(e))
+    return uri
+
+
+def _fix_uri(func):
+    def wrapper(*args, **kwargs):
+        if 'id' in kwargs:
+            id = kwargs['id']
+        else:
+            id = args[1]
+        fixed = args[0]._get_uris(id)
+        for uri in fixed:
+            if 'id' in kwargs:
+                kwargs['id'] = uri
+            else:
+                tmp = list(args)
+                tmp[1] = uri
+                args = tuple(tmp)
+            try:
+                return func(*args, **kwargs)
+            except exceptions.NotFoundException:
+                if id != uri:
+                    logger.debug(uri + ' not found.')
+        if len(fixed) > 1:
+            logger.debug('Not found any of the possible URIs: {}'.format(' '.join(fixed)))
+        raise exceptions.NotFoundException(id)
+    return wrapper
+
+
 class StorageInventoryClient(object):
     """Class to access CADC storage inventory.
 
@@ -308,6 +350,11 @@ class StorageInventoryClient(object):
             agent, retry=True, host=self.host,
             insecure=insecure)
 
+        # for now, this is only used to get the pub schema-archive mapping info
+        self._data_client = net.BaseWsClient(DATA_RESOURCE_ID, net.Subject(),
+                                             agent, retry=True, host=self.host,
+                                             insecure=insecure)
+
     @property
     def transfer(self):
         """
@@ -332,32 +379,46 @@ class StorageInventoryClient(object):
         except KeyError:
             return None
 
+    @_fix_uri
     def cadcget(self, id, dest=None, fhead=False, process_bytes=None):
         """
         Get a file from an archive. The entire file is delivered unless the
-         cutout argument is present specifying a cutout to extract from file.
+        cutout argument is present in the id in which case only the
+        specified sections of a FITS file are downloaded.
         :param id: the CADC Storage Inventory identifier (URI) of the file to
-        retrieve
+        retrieve. If the scheme in the URI is missing, the system will try to
+        guess it and return the first match. The id could also contain cutout
+        parameters if only parts of a FITS file are required ex:
+        CFHT/806045o.fits.fz?cutout=[1][10:120,20:30]&cutout=[2][10:120,20:30]
         :param dest: file to save data to (file, file_name, stream or
         anything that supports open/close and write).
         :param fhead: return the FITS header information (for all extensions)
         :param process_bytes: function to be applied to the received bytes
         """
+
         validate_uri(id)
         logger.debug('cadcget GET {} -> {}'.format(id, dest))
-        # TODO cutouts
+        params = []
+        uri = urlparse(id)
+        if 'cutout=[' in uri.query.lower():
+            lquery = uri.query.lower()
+            params = [('SUB', x.strip('&')) for x in lquery.split('cutout=')[1:]]
+            id = uri.scheme + ":" + uri.path
         # TODO transfer optimizations (skip download when destination exists)
-        urls = self._get_transfer_urls(id)
+        urls = self._get_transfer_urls(id, params=params)
         if len(urls) == 0:
             raise exceptions.HttpException('No URLs available to access data')
         last_exception = None
         for url in urls:
             if fhead:
-                # TODO this should not be allowed with cutouts
-                url = '{}?META=true'.format(url)
+                if params:
+                    raise AttributeError(
+                        'Cannot perform fhead and cutout at the same time')
+                else:
+                    params.append(('META', 'true'))
             logger.debug('GET from URL {}'.format(url))
             try:
-                self._cadc_client.download_file(url=url, dest=dest)
+                self._cadc_client.download_file(url=url, dest=dest, params=params)
                 return
             except Exception as e:
                 # try a different URL
@@ -384,7 +445,7 @@ class StorageInventoryClient(object):
         """
         Puts a file into the inventory system
         :param id: unique identifier (URI) for the file in the CADC inventory
-        system
+        system. The URI must include the scheme.
         :param src: location of the source file
         :param replace: boolean indicated whether this is expected to be
         a replacement of an existing file in the inventory system or not (file
@@ -513,7 +574,7 @@ class StorageInventoryClient(object):
         Removes a file into the inventory system. `NotFoundException` is raised
         if the `id` is not found.
         :param id: unique identifier (URI) for the file in the CADC inventory
-        system.
+        system. The URI must include the scheme.
         """
 
         # We actually raise an exception here since the web
@@ -558,12 +619,14 @@ class StorageInventoryClient(object):
             raise exceptions.NotFoundException(id)
         raise exceptions.HttpException(error_msg)
 
+    @_fix_uri
     def cadcinfo(self, id):
         """
-        Get information regarding a file in the archive
-        :param archive: Name of the archive
-        :param file_name: name of the file
-        :returns dictionary of attributes/values
+        Get information regarding a file in SI.
+        :param id: unique identifier (URI) for the file in the CADC inventory
+        system. If the scheme in the URI is missing, the system will try to
+        guess it and return the first match.
+        :returns FileInfo object with the file metadata
         """
         validate_uri(id)
         resource = (FILES_STANDARD_ID, id)
@@ -588,7 +651,7 @@ class StorageInventoryClient(object):
         logger.debug('File info: {}'.format(file_info))
         return file_info
 
-    def _get_transfer_urls(self, id, is_get=True):
+    def _get_transfer_urls(self, id, params=None, is_get=True):
         if not self.transfer:
             # this is site location
             return ['{}/{}'.format(self.files, id)]
@@ -596,7 +659,7 @@ class StorageInventoryClient(object):
         return trans.transfer(
             endpoint_url=self.transfer, uri=id,
             direction='pullFromVoSpace' if is_get else 'pushToVoSpace',
-            with_uws_job=False)
+            with_uws_job=False, cutout=params)
 
     def _get_md5sum(self, filename):
         # return the md5sum of a file
@@ -606,6 +669,49 @@ class StorageInventoryClient(object):
                 hash_md5.update(chunk)
         return hash_md5.hexdigest()
 
+    def _get_uris(self, target):
+        # takes a target URI and if the URI is not fully qualified (schema
+        # is missing, returns a list of possible fully qualified
+        # corresponding URIs
+        parts = urlparse(target)
+        if parts.scheme:
+            return [target]
+
+        scheme_file = os.path.join(
+            os.path.dirname(self._data_client.caps.caps_file),
+            '.data_uri_scheme_map')
+        scheme_url = self._data_client.caps.caps_urls[DATA_RESOURCE_ID].replace('/capabilities', '/uri-scheme-map')
+        content = self._data_client.caps._get_content(scheme_file, scheme_url)
+        schemes = self._parse_scheme_config(content)
+        archive = target.split('/')[0]
+        if archive in schemes:
+            return ['{}:{}'.format(x.strip(':'), target) for x in schemes[archive]]
+        else:
+            return ['{}:{}'.format(schemes['default'][0], target)]
+
+    def _parse_scheme_config(self, content):
+        # parses the uri schemes map. Returns dictionary of archives and
+        # corresponding list of schemes. It alwas contains the 'default` key
+        result = {}
+        for row in content.split('\n'):
+            row = row.strip()
+            if not row or row.startswith('#'):
+                continue
+            tokens = row.split(':')
+            if not tokens:
+                raise ValueError('Cannot parse archive/scheme content in row ' + row)
+            archive = tokens[0].strip()
+            if not archive:
+                raise ValueError('Cannot find archive name in row ' + row)
+            ns = ''.join(tokens[1:])
+            name_spaces = [i.strip() for i in ns.split()]
+            if not name_spaces:
+                raise ValueError('No scheme found for archive {} (row {})'.format(archive, row))
+            result[archive] = name_spaces
+        if 'default' not in result:
+            result['default'] = ['cadc']
+        return result
+
 
 def cadcput_cli():
     parser = util.get_base_parser(subparsers=False,
@@ -629,7 +735,7 @@ def cadcput_cli():
                              'is new. This is a safeguard for accidental '
                              'file replacements')
     parser.add_argument(
-        'identifier', type=argparse_validate_uri,
+        'identifier', type=argparse_validate_uri_strict,
         help='unique identifier (URI) given to the file in the CADC '
              'Storage Inventory or a root identifier when multiple files'
              'are uploaded at the same time')
@@ -701,7 +807,12 @@ def cadcget_cli():
         required=False)
     parser.add_argument(
         'identifier', type=argparse_validate_uri,
-        help='unique identifier (URI) given to the file in the CADC '
+        help='unique identifier (URI) given to the file in the CADC, typically'
+             ' of the form <scheme>:<archive>/<filename> where <scheme> is a'
+             ' concept internal to SI and is optional with this command. It is'
+             ' possible to attach cutout arguments to the identifier to'
+             ' download specific sections of a FITS file as in:'
+             ' CFHT/806045o.fits.fz?cutout=[1][10:120,20:30]'
              'Storage Inventory')
     parser.add_argument(
         '--fhead', action='store_true',
@@ -709,11 +820,11 @@ def cadcget_cli():
     parser.epilog = (
         'Examples:\n'
         '- Anonymously download a file to current directory:\n'
-        '      cadcget gemini:GEMINI/00aug02_002.fits\n'
-        '- Use certificate to get a cutout and save it to a file:\n'
-        '      cadcget --cert ~/.ssl/cadcproxy.pem -o '
-        '/tmp/700000o-cutout.fits\n'
-        '        cadc:CFHT/700000o[1]\n')
+        '      cadcget GEMINI/N20220825S0383.fits\n'
+        '- Use certificate and a full specified id to get a cutout and save '
+        'it to a file in the current directory (service provided file name):\n'
+        '      cadcget --cert ~/.ssl/cadcproxy.pem '
+        '"CFHT/806045o.fits.fz?cutout=[1][10:120,20:30]&cutout=[2][10:120,20:30]"\n')
 
     args = parser.parse_args()
     client = _create_client(args)
@@ -733,12 +844,17 @@ def cadcinfo_cli():
 
     parser.add_argument(
         'identifier', type=argparse_validate_uri,
-        help='unique identifier (URI) given to the file in the CADC '
-             'Storage Inventory', nargs='+')
+        help='unique identifier (URI) given to the file in the CADC, typically'
+             ' of the form <scheme>:<archive>/<filename> where <scheme> is a '
+             ' concept internal to the storage system and is optional with this command.',
+             nargs='+')
     parser.epilog = (
         'Examples:\n'
-        '- Anonymously getting a public file:\n'
-        '        cadcinfo gemini:GEMINI/00aug02_002.fits\n')
+        '- Anonymously getting information about a public file:\n'
+        '        cadcinfo CFHT/1000003f.fits.fz\n'
+        '- Anonymously getting the information for the same public file '
+        '  using a full URI:\n'
+        '        cadcinfo cadc:CFHT/1000003f.fits.fz\n')
 
     args = parser.parse_args()
     client = _create_client(args)
@@ -746,7 +862,8 @@ def cadcinfo_cli():
         logger.info('INFO for id {}'.format(id))
         try:
             file_info = execute_cmd(client.cadcinfo, {'id': id})
-            print('CADC Storage Inventory identifier {}:'.format(id))
+            print('CADC Storage Inventory artifact {}:'.format(id))
+            print('\t {:>15}: {}'.format('id', file_info.id))
             print('\t {:>15}: {}'.format('name', file_info.name))
             print('\t {:>15}: {}'.format('size', file_info.size))
             print('\t {:>15}: {}'.format('type', file_info.file_type))
@@ -770,7 +887,7 @@ def cadcremove_cli():
         'Remove files from the CADC Storage Inventory')
 
     parser.add_argument(
-        'identifier', type=argparse_validate_uri,
+        'identifier', type=argparse_validate_uri_strict,
         help='unique identifier (URI) given to the file in the CADC '
              'Storage Inventory', nargs='+')
     parser.epilog = (
